#!/usr/bin/env python3


import os
import unittest
import yaml
import amulet
import sys
import time
import requests

# Lots of prereqs on this charm so give it a large timeout accordingly
seconds_to_wait = 20000


class BundleTest(unittest.TestCase):
    """ Create a class for testing the charm in the unit test framework. """
    @classmethod
    def setUpClass(cls):
        """ Set up an amulet deployment using the bundle. """
        d = amulet.Deployment(series='trusty')
        local_path = os.path.join(os.path.dirname(__file__), 'local.yaml')
        with open(local_path, "r") as fd:
            config = yaml.safe_load(fd)

        host = config.get('java').get('java_host')
        print('Using Host  %s' % host)
        # Test if host for the repository is defined
        if not host:
            print("You need to define a host for the IBM Java packages repository.\n"
                  "Edit local.yaml or tests/00-setup and run it again.")
            sys.exit(1)

        package_dir = config.get('java').get('java_package_dir')
        print('Using package_dir  %s' % package_dir)
        # Test if package_dir for the repository is defined
        if not package_dir:
            print("You need to define a package_dir for the IBM Java packages"
                  " repository.\n Edit local.yaml or tests/00-setup"
                  " and run it again.")
            sys.exit(1)

        java_username = config.get('java').get('java_username')
        print('Using java_username  %s' % java_username)
        # Test if java_username for the repository is defined
        if not java_username:
            print("You need to define a java_username for the IBM Java packages"
                  " repository.\n Edit local.yaml or tests/00-setup"
                  " and run it again.")
            sys.exit(1)

        java_password = config.get('java').get('java_password')
        print('Using java_password  %s' % java_password)
        # Test if a password for the repository is defined
        if not java_password:
            print("You need to define a java_password for the IBM Java packages"
                  " repository.\n Edit local.yaml or tests/00-setup"
                  " and run it again.")
            sys.exit(1)

        license = config.get('java').get('java_accept-ibm-java-license')
        print('Using IBM Java EULA value %s' % license)
        # Test if EULA value for the repository is defined
        if not license:
            print("You need to define a path name for IBM Java.\n"
                  "Edit local.yaml or tests/00-setup and run it again.")
            sys.exit(1)

        itype = config.get('java').get('java_install-type')
        print('Using IBM Java Install type value %s' % itype)
        # Test if an itype for ibm java installation is defined
        if not itype:
            print("You need to define installtion type for IBM Java.\n"
                  "Edit local.yaml or tests/00-setup and run it again.")
            sys.exit(1)

        d.add('ibm-java')
        d.add('ubuntu-devenv', 'cs:~kwmonroe/trusty/ubuntu-devenv-1')

        # Software doesn't actually install until you accept the license
        d.configure('ibm-java', {'accept-ibm-java-license': license,
                                 'host': host,
                                 'package_dir': package_dir,
                                 'username': java_username,
                                 'password': java_password,
                                 'install-type': itype})
        d.relate('ibm-java:java', 'ubuntu-devenv:java')
        time.sleep(120)
        d.setup(seconds_to_wait)
        d.sentry.wait(seconds_to_wait)
        cls.d = d

    def test_deployed(self):
        """ Test to see if the bundle deployed successfully. """
        self.assertTrue(self.d.deployed)
        ibmjava_unit = self.d.sentry['ibm-java'][0]
        local_path = os.path.join(os.path.dirname(__file__), 'local.yaml')
        with open(local_path, "r") as fd:
            config = yaml.safe_load(fd)
        itype = config.get('java').get('java_install-type')
        if itype == "jre":
            # Create a simple java program
            cmd = "java -version"
            output, code = ibmjava_unit.run(cmd)
            if code != 0:
                message = 'Unable to run Java Command: As java is not installed or appropriate path variable is not set.'
                amulet.raise_status(amulet.FAIL, msg=message)
        else:
            ibmjava_unit.run('mkdir -m a=rwx /home/test')
            ibmjava_unit.run('echo "import java.util.Date;" >> /home/test/Simple.java')
            ibmjava_unit.run('echo "public class Simple{" >> /home/test/Simple.java')
            ibmjava_unit.run('echo "public static void main(String args[]){" >> /home/test/Simple.java')
            ibmjava_unit.run('echo "Date date = new Date();" >> /home/test/Simple.java')
            ibmjava_unit.run('echo "System.out.println(date.toString());" >> /home/test/Simple.java')
            ibmjava_unit.run('echo "}" >> /home/test/Simple.java')
            ibmjava_unit.run('echo "}" >> /home/test/Simple.java')
            ibmjava_unit.run('chmod +x Simple.java')
            ibmjava_unit.run('/opt/ibm/java/bin/javac -d /home/test/ /home/test/Simple.java')
            time.sleep(120)
            cmd = "java -cp /home/test Simple"
            output, code = ibmjava_unit.run(cmd)
            print('Output of simple java program is  %s' % output)
            if code != 0:
                message = 'Unable to run Java Program: As java is not installed or appropriate path variable is not set.'
                amulet.raise_status(amulet.FAIL, msg=message)

if __name__ == '__main__':
    unittest.main()
